@@ -2,8 +2,7 @@
 # -*- coding: utf-8 -*-
 '''Object wrapper for key-value db'''
 import os
-import zlib
-import shutil
+import sys
 import marshal
 import time
 from datetime import datetime, date
@@ -12,10 +11,6 @@ from datetime import datetime, date
 md = marshal.dumps
 ml = marshal.loads
 
-class DummyDB(dict):
-    def save(self):
-        pass
-
 class class_cached_property(object):
     '''
     Кеширующий classmethod
@@ -28,32 +23,43 @@ class class_cached_property(object):
         setattr(owner, self.f.__name__, value)
         return value
 
-
 class Model(object):
     '''
     >>> class Test(Model):
-    ...     __db_type__ = 'dict'
-    ...     foo = DateProperty(primary_key=True)
-    ...     bar = Property(default=[])
-    ...     baz = UnicodeProperty(key='d')
+    ...     day     = DateProperty(primary_key=True)
+    ...     foo     = Property(default=[1, 2])
+    ...     ustr    = UnicodeProperty(key='u')
+    ...     cstr    = CompressedStringProperty()
+    ...     cuni    = CompressedUnicodeProperty() 
+    ...     cdump   = CompressedProperty(default={}) 
     ...     created = DateTimeProperty(key='c')
-    >>> Test(
-    ...     foo=date(2000, 01, 02),
-    ...     bar=[1, 2], baz=u'ыыы',
-    ...     created=datetime.now()
-    ... ).save()
-    >>> Test(foo=date(1999, 02, 02), created=datetime.now()).save()
+    ...
+    ...     __db_type__ = 'dict'
     
-    >>> Test.get(date(2000, 01, 02)).bar
+    >>> Test(
+    ...     day     = date(2000, 01, 02),
+    ...     foo     = [1, u'ы', {'a': 'b'}],
+    ...     ustr    = u'ыыы',
+    ...     cstr    = 'A' * 1000,
+    ...     cuni    = u'Ы' * 1000,
+    ...     cdump   = range(10),
+    ...     created = datetime.now(),
+    ... ).save()
+    >>> Test(day=date(1999, 02, 02)).save()
+    
+    >>> Test.get(date(1999, 02, 02)).foo
     [1, 2]
+    
     >>> Test.count()
     2
-    >>> Test.count(lambda x: x.baz == u'ыыы')
+    >>> Test.count(lambda x: x.ustr == u'ыыы')
     1
-    >>> [t.foo.year for t in Test.find(
-    ...     filter  = lambda x: 3 not in x.bar,
-    ...     order   = lambda x: x.foo.year)]
+    
+    >>> [t.day.year for t in Test.find(
+    ...     filter  = lambda x: 3 not in x.foo,
+    ...     order   = lambda x: x.day)]
     [1999, 2000]
+    
     >>> Test.find_one().delete()
     >>> Test.count()
     1
@@ -147,14 +153,19 @@ class Model(object):
             ret = tc.HDB()
             ret.open(cls.__filename__, tc.HDBOWRITER | tc.HDBOCREAT)
             return ret
-        elif cls.__db_type__ == 'fdb':
-            import om.fdb
-            return om.fdb.FDB(cls.__filename__)
+        elif cls.__db_type__ == 'kc':
+            import kyotocabinet as kc
+            ret = kc.DB()
+            ret.open("var/test.kch", db.OWRITER | db.OCREATE)
+            return ret
+        elif cls.__db_type__ == 'fsdbm':
+            import fsdbm
+            return fsdbm.FSDBM(cls.__filename__)
         elif cls.__db_type__ == 'gdbm':
             import gdbm
             return gdbm.open(cls.__filename__, 'c')
         elif cls.__db_type__ == 'dict':
-            return DummyDB()
+            return {}
         assert 0, 'this db is not supported'
     
     @class_cached_property
@@ -169,9 +180,18 @@ class Model(object):
         '''
         {prop_attr_name: save_key_name}
         '''
+        def req_bases(cls):
+            bases = list(cls.__bases__) or []
+            ret = []
+            for base in bases:
+                for c in req_bases(base):
+                    if c not in ret and c not in bases:
+                        ret.append(c)
+            ret.extend(bases)
+            return ret        
         ret = {}
         for k, prop in cls.__dict__.iteritems():
-            if BaseProperty in prop.__class__.__bases__:
+            if BaseProperty in req_bases(prop.__class__):
                 if not prop.key:
                     prop.key = k
                 if prop.key in ret.values():
@@ -259,6 +279,44 @@ class DateProperty(BaseProperty):
         return date.fromtimestamp(value)
 
 
+class CompressedStringProperty(BaseProperty):
+    def __init__(self, compressor='zlib', compress_level=9, *args, **kwargs):
+        __import__(compressor)
+        self.compressor = sys.modules[compressor]
+        self.compress_level = compress_level
+        super(CompressedStringProperty, self).__init__(*args, **kwargs)
+    def encode(self, value):
+        return self.compressor.compress(value, self.compress_level)
+    def decode(self, value):
+        return self.compressor.decompress(value)
+
+
+class CompressedUnicodeProperty(CompressedStringProperty):
+    def __init__(self, internal_encoding='utf-8',
+                encoding_errors='ignore', *args, **kwargs):
+        self.internal_encoding = internal_encoding
+        self.encoding_errors = encoding_errors
+        super(CompressedUnicodeProperty, self).__init__(*args, **kwargs)
+    def encode(self, value):
+        value = value.encode(self.internal_encoding, self.encoding_errors)
+        return super(CompressedUnicodeProperty, self).encode(value)
+    def decode(self, value):
+        value = super(CompressedUnicodeProperty, self).decode(value)
+        return value.decode(self.internal_encoding, self.encoding_errors)
+
+
+class CompressedProperty(CompressedStringProperty):
+    def __init__(self, dumper='marshal', *args, **kwargs):
+        __import__(dumper)
+        self.dumper = sys.modules[dumper]
+        super(CompressedProperty, self).__init__(*args, **kwargs)
+    def encode(self, value):
+        value = self.dumper.dumps(value)
+        return super(CompressedProperty, self).encode(value)
+    def decode(self, value):
+        value = super(CompressedProperty, self).decode(value)
+        return self.dumper.loads(value)
+
 
 if __name__ == "__main__":
     import doctest
